import httpx
from .enums import API, ExceptionMsg, Joker
from typing import Any
from .exceptions import *

class SDK:
    def __init__(self, api_key: str = None, **kwargs) -> None:
        self.api_key: str = api_key
        return
    
    def DIAL(self,
             dial_to: str,
             dial_from: str,
             callback_url: str,
             proxy: str = {},
             request_method: str = "GET"
             ) -> dict:
        
        if not self.api_key:
            raise REQUIREDAPIKEY(ExceptionMsg.REQUIRED_KEY)
        
        response: dict = httpx.request(request_method.upper() if request_method.upper() in ["GET", "POST"] else "GET", API.__translator__("DIAL", API.URL), proxies=proxy if proxy != {} else None).json()

        if response['status'] == Joker.INVALIDAUTH:
            self.api_key = None
            raise INVALIDAPIKEY(ExceptionMsg.INVALID_KEY)
        


__context__: tuple = (
    "API.INVALID",
    "API.VALID"
)


from typing import Any
from strenum import StrEnum

class ExceptionMsg:
    REQUIRED_KEY = "Please set a API key before processing to dial or control the flow of a call."
    INVALID_KEY = "The API key you've provided is incorrect, you may not be able to dial, or control the flow of a call without it."

class API(StrEnum):
    URL = "https://api.jokerapi.co/voice/v1/{}"
    def __translator__(val: str, url) -> str:
        return {
            "DIAL": url.format("dial"),
            "PLAY": url.format("play"),
            "PLAYTEXT": url.format("playtext"),
            "gAUDIO": url.format("gather"),
            "gText": url.format("gathertext"),
            "TRANSFER": url.format("transfer"),
            "HANGUP": url.format("hangup")
            }[val]

class Joker(StrEnum):
    """
    An enumeration of responses registered and heard from JokerAPI.
        Used in library to handle differed responses to be read.
    """
    __unknown__ = []
    INVALIDAUTH =  "invalid api key"
    INVALIDPARAM = "you are missing parameters"
    SERVERFAILURE = "Connection failed"
    NO_BALANCE = "you have no balance"
    INVALIDCOUNTRY = "this country is not whitelisted"
    
    GATHER_AUDIO = "playing audio while gathering"
    GATHER_TEXT = "playing text while gathering"

    CALL_ENDED = "call ended"

    PLAY_AUDIO = "playing audio"
    
    PLAY_TEXT = "playing text"
    
    TRANSFERING = "transfering call"

    def __INVALID__(value: str) -> list[Any, int]:
        return (_ := Joker.__unknown__.append([value])), len(Joker.__unknown__)[-1]


import os
from setuptools import setup

setup(
    name="JokerSDK",
    version="0.0.3",
    author="0x96e63",
    description=open("README.md", "r", encoding="UTF-8").read(),
    url="https://github.com/0x96e63/JokerAPI-SDK",
    packages=[
        os.path.join(root).replace("\\", ".") for root, _, files in os.walk("JokerAPI") if "__init__.py" in files
    ]
)